package ch.example.java.pkcs11.config;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStoreException;
import java.util.Optional;
import java.util.OptionalLong;

import sun.security.pkcs11.wrapper.PKCS11;
import sun.security.pkcs11.wrapper.PKCS11Exception;

/**
 * Basic PKCS#11 integration.
 * 
 * @author Christoph
 * @see <a href="http://docs.oracle.com/javase/7/docs/technotes/guides/security/p11guide.html">Java PKCS#11 Reference Guide</a>
 *
 */
public class ConfigFactory {
	private final String libraryPath;
	
	/**
	 * Creates a new instance of the factory with the given PKCS#11
	 * library path.
	 * 
	 * @param libraryPath The path to the PKCS#11 library.
	 */
	public ConfigFactory(String libraryPath) {
		this.libraryPath = libraryPath;
	}
	
	/**
	 * Creates a PKCS#11 configuration with a slot which has a token inserted.
	 * 
	 * @return The PKCS#11 configuration.
	 * @throws PKCS11Exception 
	 * @throws IOException 
	 */
	@SuppressWarnings("restriction")
	public Optional<InputStream> createConfig() throws IOException, PKCS11Exception {
		OptionalLong slot = getSlot();
		if(!slot.isPresent()) {
			return Optional.empty();
		}
		
		return Optional.of(createConfig(slot.getAsLong()));
	}
	
	private InputStream createConfig(long slot) {
		StringBuilder buffer = new StringBuilder();
		buffer.append("name = Pkcs11Accelerator\r\nlibrary = ");
		buffer.append(libraryPath);
		buffer.append("\r\nslot = ");
		buffer.append(slot);
		buffer.append("\r\n");
		return new ByteArrayInputStream(buffer.toString().getBytes());
	}

	@SuppressWarnings("restriction")
	private OptionalLong getSlot() throws IOException, PKCS11Exception {
		PKCS11 pkcs11 = PKCS11.getInstance(libraryPath, "C_GetFunctionList", null, false);
		long[] slots = pkcs11.C_GetSlotList(true);
		if(slots.length > 0) {
			return OptionalLong.of(slots[0]);
		}
		return OptionalLong.empty();
	}
}
